
#ifndef KORNRES_SCENE
#define KORNERS_SCENE

#include "korners_game_engine.h"
#include "korners_time_line.h"

#include <QGraphicsScene>
#include <QTime>
#include <QTimer>

class KGamePopupItem;
class KornersCapItem;
class KornersEngineThread;


/**
 *  Displays and drives the game
 */
class KornersScene : public QGraphicsScene
{
    Q_OBJECT
public:
    explicit KornersScene( QObject *parent = 0 );
    ~KornersScene() {}

    /**
     *  Resizes scene
     */
    void resizeScene( int width, int height );
    /**
     *  Starts new game
     */
    void startNetworkGameClient( const QString& net_name, int port, const QString& ip_adress );
    void startNetworkGameServer( const QString& net_name, int port );
    void startNewLocalGame();
    /**
     *  Toggles time game
     */
    void toggleOnTime();

public slots:

    /**
     *  Animate route
     */
    void showMovePath( KornersMove mv );
    /**
     *  Highlight possible routes
     */
    void showPaths( const QVector<QPoint>* moves );
    /**
     *  Opponent name received in network game
     */
    void opponentName( QString );
    /**
     *  Show popup notification from game engine
     */
    void showEngineMess( QString text );
    /**
     *  Your turn
     */
    void canPlay();
    /**
     *  You won or loose
     */
    void youWin();
    void youLost();

signals:
    /**
     *  Start different game engines
     */
    void startGameEngineAI();
    void startGameEngineNetwork( const QString& net_name, int port, bool server = false,
                                 const QString& ip_adress = "localhost" );
    /**
     *  Notify about players move
     */
    void playerMadeMove( QPoint , QPoint );
    /**
     *  Get moves available to player
     */
    void getMovesFromPoint( QPoint );
    /**
     *  Game over (somebody won or lost)
     */
    void gameOver( QTime );
    /**
     *  New status text to show
     */
    void newStatus1( QString );
    void newStatus2( QString );
    /**
     *  Player made first click
     */
    void started();

private slots:
    void moveAnimationFrameChanged(int frame);
    void moveAnimFinished();

private:

    /**
     *  Field coords to pixel coords
     */
    inline QPointF fieldToPix(int xField, int yField) const
    {
        return QPointF( m_playFieldRect.x() + m_playFieldBorderSize + xField*m_cellSize,
                        m_playFieldRect.y() + m_playFieldBorderSize + yField*m_cellSize );
    }
    /**
     *  Pixel coords to field coords
     */
    inline QPoint pixToField( const QPointF& p ) const
    {
        return QPoint(static_cast<int>(( p.x()-m_playFieldRect.x()-m_playFieldBorderSize )/m_cellSize),
                      static_cast<int>(( p.y()-m_playFieldRect.y()-m_playFieldBorderSize )/m_cellSize));
    }
    /**
     *  Reset scene
     */
    void putCapsInStartPos();
    void resetScene();
    /**
     *  Event handlers
     */
    virtual void drawBackground( QPainter*, const QRectF& );
    virtual void mousePressEvent( QGraphicsSceneMouseEvent* );
    virtual void timerEvent( QTimerEvent* event );
    /**
     *  Hide route highlighter
     */
    void unshowPaths();
    /**
     *  Begins move animation
     */
    bool startMoveAnimation(const KornersMove& mv);
    /**
     *  If we selected cap
     */
    bool m_capSelected;
    /**
     *  Player's move
     */
    QPoint point_source;
    QPoint point_destination;
    /**       
     * Area of playfield (with border included - if any exists in theme)
     */
    QRect m_playFieldRect;
    /**
     * Size of a playfield border.
     * Equals 0 if there's no border element in current theme
     */
    int m_playFieldBorderSize;
    /**
     *  Cell size in pixels
     */
    int m_cellSize;
    /**
     * Item to show popup messages to user
     */
    KGamePopupItem* m_popupItem;
    /**
     *  If the game is over
     */   
    bool m_gameOver;
    /**
     *  If you made first mouse click
     */
    bool m_gameStarted;
    /**
     *  If you can select caps
     */
    bool m_canPlay;
    /**
     *  If you play time game
     */
    bool m_timeGame;
    /**
     *  Move animation timeline
     */
    KornersTimeLine m_moveTimeLine;
    /**
     *  List of routes to animate
     */
    QVector<KornersMove> m_pendingMoves;
    /**
     *  Current moving cap
     */
    KornersCapItem* m_curMovingCap;
    /**
     *  Opponents name in network game
     */
    QString m_name;

    /**
     *  Class to keep game time
     */
    class KornersTime
    {
    public:
        KornersTime()
        {
            m_value = 0;
            m_started = false;
        }
        void start()
        {
            if( !m_started )
            {
                m_time.start();
                m_started = true;
            }
        }
        void restart()
        {
            m_time.restart();
        }
        void stop()
        {
            m_value += m_time.elapsed();
        }
        QTime value()
        {
            return QTime( 0, (m_value/1000)/60, (m_value/1000)%60, m_value%1000 );
        }

    private:
        bool m_started;
        long m_value;
        QTime m_time;
    };

    /**
     *  Time counter
     */
    KornersTime m_time;
};

#endif
